<?php

namespace Teknisa\FIN\Modules\ContaPagar\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Teknisa\FIN\Controllers\CrudService;
use Zeedhi\Framework\DTO;
use Geral\Util\DateUtil;
use Teknisa\FIN\Modules\ContaPagar\Services\Pag16000EmprestimoLeasingPagarService;

class Pag16000EmprestimoLeasingPagarController extends TeknisaCrud{

    protected $dataSourceName;
    protected $services;

    public function __construct($crudService, $parameterBag, Pag16000EmprestimoLeasingPagarService $services){
        $this->services = $services;
        parent::__construct($crudService, $parameterBag);
    }

    public function saveEmprestimoPagar(DTO\Request $request, DTO\Response $response){
        try{
            set_time_limit(0);
            $rows = $request->getDataSet()->getRows();
            $rowEmprestimo = $rows[0];
            $dataSetParcelas = $rows[1];
            $rowCredito = $rows[2];
            $this->services->saveEmprestimoPagar($rowEmprestimo, $dataSetParcelas, $rowCredito, $response);
        }catch(\Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

     public function saveReparcelamentoEmprestimoPagar(DTO\Request $request, DTO\Response $response){
        try{
            $rows = $request->getDataSet()->getRows();
            $emprestimo = $rows[0];
            $parcelasExclusao = $rows[1];
            $dataSetParcelas = $rows[2];
            $this->services->saveReparcelamentoEmprestimoPagar($emprestimo, $parcelasExclusao, $dataSetParcelas);
        }catch(\Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function saveCreditoEmprestimoPagar(DTO\Request $request, DTO\Response $response){
        try{
            $credito = $request->getRow();
            $this->services->saveCreditoEmprestimoPagar($credito);
        }catch(\Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function deleteDocumentoEmprestimoPagar(DTO\Request $request, DTO\Response $response){
        try{
            $rows = $request->getDataSet()->getRows();
            $emprestimoPagar = $rows[0];
            $dataSetParcelas = $rows[1];
            $credito = (array_key_exists("2", $rows)) ? $rows[2] : null;
            $debito = (array_key_exists("3", $rows)) ? $rows[3] : null;
            $this->services->deleteDocumentoEmprestimoPagar($emprestimoPagar, $dataSetParcelas, $credito, $debito, $response);
        }catch(\Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function deleteParcelaEmprestimoPagar(DTO\Request $request, DTO\Response $response){
        try{
            $rows = $request->getDataSet()->getRows();
            $this->services->deleteParcelaEmprestimoPagar($rows);
        }catch(\Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function deleteReparcelamentoEmprestimoPagar(DTO\Request $request, DTO\Response $response){
        try{
            $rows = $request->getDataSet()->getRows();
            $this->services->deleteReparcelamentoEmprestimoPagar($rows);
        }catch(\Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function aplicarMascaraInscricao(DTO\Request\Filter $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'fornecedor_maskIsncricaoFornecedor';
            parent::find($request, $response);
            $dataSets = $response->getDataSets();
            if(!empty($dataSets)) {
                $rows = $dataSets[0]->getRows();
                $dados = $this->services->aplicarMascaraInscricao($rows);
                $dataSet = new DataSet($this->dataSourceName, $dados);
                $response->setDataSets(array($dataSet));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function verificaGrupoOperadores(DTO\Request $request, DTO\Response $response){
        try{
            $statusGrupo = $this->services->verificaGrupoOperadores();
            $response->addDataSet(new DataSet("STATUS", [$statusGrupo]));
        }catch(\Exception $e){
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
        }
    }

    public function validaPermissaoEdicaoTitulo(DTO\Request $request, DTO\Response $response){
            $row = $request->getRow();
            $this->services->validaPermissaoEdicaoTitulo($row);
    }

    public function geraNovoCodigo(Row $row, Response $response)
    {
        try {
            // Dados do Registro
            $row  = $row->getRow();

            $nrorg      = $row['NRORG'];
            $cdcontador = $row['CDCONTADOR'];

            $novocodigo = $this->services->geraNovoCodigo($nrorg, $cdcontador);

            $response->addDataSet(new DataSet('geraNovoCodigo', ['novocodigo' => $novocodigo]));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}