<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services\Retorno;

use \Geral\Entities\Empresa;
use \Geral\Util\Ambiente;
use Fiscal\Entities\Boletpagar;

class LeitorCnabSicredi extends LeitorCnab{

    CONST SEGMENTO_A = 'A';
    CONST SEGMENTO_J = 'J';
    
    CONST SEGMENTO_O = 'O';
    CONST SEGMENTO_N = 'N';
    
    CONST ID_REGISTRO_HEADER = '0';
    CONST ID_REGISTRO_LOTE = '1';
    CONST ID_REGISTRO_TRANSACAO = '3';
    CONST ID_REGISTRO_TRAILLER_LOTE = '5';
    CONST ID_REGISTRO_TRAILLER = '9';

    CONST TIPO_MOVIMENTO_INCLUSAO = 0;
    CONST TIPO_MOVIMENTO_ALTERACAO = 5;
    CONST TIPO_MOVIMENTO_EXCLUSAO = 9;
    CONST CODIGO_COMPENSACAO_SICREDI = '748';

    private $tpProcessamento;
    private $cdagencia;
    private $contcorr;
    private $ocorrHeader;
    private $nrlanbol = 0;
    private $cdfilial = 0;

    public function __construct($arquivo, $cdempresa, $cdbanco){
        try {

            $this->cdbanco = $cdbanco;
            $this->arquivo = $arquivo;
            $this->cdempresa = $cdempresa;

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $linha
     * @return bool
     */
    private function validaArquivo($linha){
        parent::setStatusArquivo(true);
        $idReg = substr($linha,7,1);
        if($this->contlinha == 1 and $idReg != self::ID_REGISTRO_HEADER ){
            self::gravaErros($this->contlinha, "Erro de identificação do registro header.",true);
            throw new Exception("Erro de identificação do registro header.", 1);
            parent::setStatusArquivo(false);
        }
        if($this->contlinha > 1 and (! in_array($idReg,
            array(self::ID_REGISTRO_HEADER, self::ID_REGISTRO_LOTE,
                self::ID_REGISTRO_TRANSACAO, self::ID_REGISTRO_TRAILLER_LOTE,
                self::ID_REGISTRO_TRAILLER))
        )){
            self::gravaErros($this->contlinha,"Identificação do registro não é válida",true);
            throw new Exception("Identificação do registro não é válida.", 1);

            parent::setStatusArquivo(false);
        }
        return parent::getStatusArquivo();
    }

    public function leArquivo($nrorg)
    {
        try {
            foreach ($this->arquivo as $linha) {
                $this->contlinha++;
                if($linha){
                    if (strlen($linha) > 0){
                        $this->retorno = new Retorno();

                        // Busca o Tipo de Registro do Arquivo
                        $idRegistro = substr($linha,7,1);
                        if(self::validaArquivo($linha)){
                            switch ($idRegistro) {
                                case(self::ID_REGISTRO_HEADER):
                                    self::leHeaderArquivo($linha);
                                    break;

                                case(self::ID_REGISTRO_TRANSACAO):
                                    self::leRegistroDetalhe($linha, $nrorg);
                                    break;

                                case(self::ID_REGISTRO_LOTE):
                                    self::leRegistroLote($linha);
                                    break;

                                case(self::ID_REGISTRO_TRAILLER_LOTE):
                                    self::leRegistroTraillerLote($linha);
                                    break;
                            }
                        }
                        $this->retornos[] = $this->retorno;
                    }
                }
            }
            parent::preparaOcorrencias($this->cdbanco, $nrorg);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    // =====================================================================  //
    // ====================      REGISTRO HEADER        ====================  //
    // =====================================================================  //
    /**
     * @param $header
     */
    private function leHeaderArquivo($header){

        // CAMPO: Código do BCO na compensação
        // POSIÇÃO: 1 a 3
        // PICTURE: 9(3)
        if (substr($header, 0, 3) != self::CODIGO_COMPENSACAO_SICREDI){
            self::gravaErros($this->contlinha, "Código do banco inválido.",true);
            parent::setStatusArquivo(false);
        }

        // CAMPO: Tipo de inscrição da empresa
        // POSIÇÃO: 18 a 18
        // PICTURE: 9(1)
        $idInsc = trim(substr($header, 17, 1));
        if (empty($idInsc)){
            self::gravaErros($this->contlinha, "Tipo de incrição da empresa não informado",true);
            parent::setStatusArquivo(false);
        }

        // CAMPO: Código Remessa/Retorno
        // POSIÇÃO: 143 a 143
        // PICTURE: 9(1)
        if(substr($header, 142, 1) != '2'){
            self::gravaErros($this->contlinha, "Arquivo de retorno inválido.", true);
            parent::setStatusArquivo(false);
        }
    }

    // =====================================================================  //
    // ====================  REGISTRO HEADER  DE LOTE   ====================  //
    // =====================================================================  //
    private function leRegistroLote($registro){
        // CAMPO: Número da agência centralizadora
        // POSIÇÃO: 53 a 57
        // PICTURE: 9(4)
        $this->cdagencia = substr($registro, 52, 5);

        // CAMPO: Conta debitada
        // POSIÇÃO: 59 a 70
        // PICTURE: 9(9)
        $this->contcorr = substr($registro, 58, 12);

        // CAMPO: Ocorrências
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $this->ocorrHeader = substr($registro, 230, 10);
    }

    // =====================================================================  //
    // ====================      REGISTRO DETALHE       ====================  //
    // =====================================================================  //
    private function leRegistroDetalhe($registro, $nrorg){
        
        $segmento = substr($registro,13,1);
        switch($segmento){
            case 'A': $this->segmentoA($registro);
                break;
            case 'J': $this->segmentoJ($registro);
                break;
            case 'O': $this->segmentoO($registro);
                break;
            case 'N': $this->segmentoN($registro);
                $this->validaFormaPagamento($registro);
                break;
        }
        
        if(($segmento == self::SEGMENTO_A) or ($segmento == self::SEGMENTO_J) or ($segmento == self::SEGMENTO_O) or ($segmento == self::SEGMENTO_N)){
            $nrlanbolpag = str_pad(trim($this->nrlanbol), 10, "0", STR_PAD_LEFT);
            $this->retorno->setCdfilial(trim($this->cdfilial));
            $this->retorno->setNrlanbolpag($nrlanbolpag);
            $this->retorno->setBoleto('BOLETO');
            $this->retorno->setCdconta(trim($this->contcorr));
            $this->retorno->setCdagencia($this->cdagencia);
            $this->retorno->setCdlinha(str_pad($this->contlinha,'0',STR_PAD_LEFT));
            $this->retorno->setCdtpbaixapag(self::getTipoBaixaPagamento($this->cdfilial, $this->nrlanbol, $nrorg));
            $this->retorno->setCdreducont(self::getReducont($this->cdfilial, $this->nrlanbol, $nrorg));
            $boletpagar = Boletpagar::findOneByCdfilialAndNrlanbolpagAndNrorg($this->cdfilial, $this->nrlanbol, $nrorg);
            $valorFinalPagoBoleto = ($boletpagar) ? $boletpagar->getVrfinalbpag() : 0;
            $this->retorno->setVrfinalpago($valorFinalPagoBoleto);
            // CAMPO: Tipo de movimento   [0 - Inclusão  9 - Exclusão]
            // POSIÇÃO: 15 a 17
            // PICTURE: 9(3)
            $this->retorno->getTpmovimento(substr($registro, 14, 3));
            $this->retorno->setOcorrencia(substr($registro, 230, 10));
            $ocorrencia = trim(substr($registro, 230, 10));
            $this->retorno->setIdsitucnabp(self::situacaoCnab($this->retorno->getOcorrencia().$ocorrencia.$this->ocorrHeader));
        }
        
    }

    private function segmentoA($registro){
        $this->cdfilial = substr($registro, 73, 4);
        $this->nrlanbol = substr($registro, 77, 10);
        $this->retorno->setNossonumero(substr($registro, 134, 15));
        $this->retorno->setDtvencimento(substr($registro, 93, 8));
        $this->retorno->setDtpagamento(substr($registro, 154, 8));
        $vrpago = floatval(substr($registro, 119, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        $nrRetorno = trim(substr($registro, 134, 15));
        if (empty($nrRetorno))
            self::gravaErros($this->contlinha, 'Número do retorno inválido.', false);
            parent::setStatusArquivo(false);
    }

    private function segmentoJ($registro){
        $this->cdfilial = substr($registro, 182, 4);
        $this->nrlanbol = substr($registro, 186, 10);
        $this->retorno->setNossonumero(substr($registro, 215, 15));
        $this->retorno->setDtvencimento(substr($registro, 91, 8));
        $vrdesconto = floatval(substr($registro, 114, 15));
        $this->retorno->setVrdesconto(round($vrdesconto / 100, 2));
        $this->retorno->setVracrescimo('0');
        $this->retorno->setDtpagamento(substr($registro, 144, 8));
        $vrpago = floatval(substr($registro, 152, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        $nrRetorno = trim(substr($registro, 215, 15));
        if (empty($nrRetorno)){
            self::gravaErros($this->contlinha, 'Número do retorno inválido.', false);
            parent::setStatusArquivo(false);
        }
    }

    private function segmentoO($registro){
        $cdfilial = substr($registro, 174, 4);
        $nrlanbol = substr($registro, 178, 10);
        $this->retorno->setNossonumero(substr($registro, 215, 15));
        $this->retorno->setDtvencimento(substr($registro, 91, 8));
        $this->retorno->setDtpagamento(substr($registro, 136, 8));
        $vrpago = floatval(substr($registro, 121, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        $nrRetorno = trim(substr($registro, 215, 15));
        if (empty($nrRetorno)){
            self::gravaErros($this->contlinha, 'Número do retorno inválido.', false);
            parent::setStatusArquivo(false);
        }
    }

    private function segmentoN($registro){
        $this->cdfilial = substr($registro, 195, 4);
        $this->nrlanbol = substr($registro, 199, 10);
        $this->retorno->setNossonumero(substr($registro, 215, 15));
        //DARF SIMPLES OU DARF NORMAL
        if (substr($registro, 17, 2) == '02' or substr($registro, 17, 2) == '03') {
            $this->retorno->setDtvencimento(substr($registro, 119, 8));
        }
        //DARJ
        else if (substr($registro, 17, 2) == '04') {
            $this->retorno->setDtvencimento(substr($registro, 133, 8));
        }
        //GARE
        else if (substr($registro, 17, 2) == '05') {
            $this->retorno->setDtvencimento(substr($registro, 138, 8));
        }
        //FGTS
        else if (substr($registro, 17, 2) == '11') {
            $this->retorno->setDtvencimento(substr($registro, 143, 8));
        }
        //GPS E OUTROS
        else {
            $this->retorno->setDtvencimento(substr($registro, 99, 8));
        }
       $this->retorno->setDtpagamento(substr($registro, 99, 8));
       $vrpago = floatval(substr($registro, 85, 15));
       $this->retorno->setVrpago(round($vrpago / 100, 2));
       // CAMPO: Acréscimo/Juros /Multa
       // POSIÇÃO: 78 a 91 / 92 a 105
       // PICTURE: 9(12)V9(2)
       //vrMultaJuro := 0;
       $this->retorno->setVracrescimo('0');
    }

    private function validaFormaPagamento($registro){
        $tipoPagamento = substr($registro, 17, 2);
        switch($tipoPagamento){
            case '01': $this->gps($registro);
                break;
            case '02': $this->darfSimplesNormal($registro);
                break;
            case '03': $this->darfSimplesNormal($registro);
                break;
            case '04': $this->darj($registro);
                break;
            case '05': $this->gare($registro);
                break;
            case '11': $this->fgts($registro);
                break;
            default: $this->tipoPagamento($registro);
                break;
        }
    }

    private function gps($registro){

    }

    private function darfSimplesNormal($registro){
        // CAMPO: Data do pagamento
        // POSIÇÃO: 128 a 135
        // PICTURE: 9(8)
        $this->retorno->setDtpagamento(substr($registro, 127, 8));
        // CAMPO: Valor do pagamento
        // POSIÇÃO: 106 a 119
        // PICTURE: 9(12)V9(2)
        $vrpago = floatval(substr($registro, 105, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        // CAMPO: Acréscimo/Juros /Multa
        // POSIÇÃO: 78 a 91 / 92 a 105
        // PICTURE: 9(12)V9(2)
        $this->retorno->setVracrescimo('0');
    }

    private function darj($registro){
        // CAMPO: Data do pagamento
        // POSIÇÃO: 128 a 135
        // PICTURE: 9(8)
        $this->retorno->setDtpagamento(substr($registro, 141, 8));
        // CAMPO: Valor do pagamento
        // POSIÇÃO: 106 a 119
        // PICTURE: 9(12)V9(2)
        $vrpago = floatval(substr($registro, 119, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        // CAMPO: Acréscimo/Juros /Multa
        // POSIÇÃO: 92 a 105 / 106 a 119
        // PICTURE: 9(12)V9(2)
        $this->retorno->setVracrescimo('0');
    }

    private function gare($registro){
        // CAMPO: Data do pagamento
        // POSIÇÃO: 147 a 154
        // PICTURE: 9(8)
        $this->retorno->setDtpagamento(substr($registro, 146, 8));
        // CAMPO: Valor do pagamento
        // POSIÇÃO: 125 a 138
        // PICTURE: 9(12)V9(2)
        $vrpago = floatval(substr($registro, 124, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        // CAMPO: Acréscimo/Juros /Multa
        // POSIÇÃO: 97 a 110 / 111 a 124
        // PICTURE: 9(12)V9(2)
        $this->retorno->setVracrescimo('0');
    }

    private function fgts($registro){
        // CAMPO: Data do pagamento
        // POSIÇÃO: 144 a 151
        // PICTURE: 9(8)
        $this->retorno->setDtpagamento(substr($registro, 143, 8));
        // CAMPO: Valor do pagamento
        // POSIÇÃO: 152 a 165
        // PICTURE: 9(12)V9(2)
        $vrpago = floatval(substr($registro, 151, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        // CAMPO: Acréscimo/Atualização Monetária
        // POSIÇÃO: ----
        // PICTURE: ----
        $this->retorno->setVracrescimo('0');
    }

    private function tipoPagamento($registro){
        // CAMPO: Data do pagamento
        // POSIÇÃO: 100 a 107
        // PICTURE: 9(8)
        $this->retorno->setDtpagamento(substr($registro, 99, 8));
        // CAMPO: Valor do pagamento
        // POSIÇÃO: 86 a 99
        // PICTURE: 9(12)V9(2)
        $vrpago = floatval(substr($registro, 105, 15));
        $this->retorno->setVrpago(round($vrpago / 100, 2));
        // CAMPO: Acréscimo/Atualização Monetária
        // POSIÇÃO: 72 a 85
        // PICTURE: 9(12)V9(2)
        $this->retorno->setVracrescimo('0');
    }

    // =====================================================================  //
    // ====================  REGISTRO TRAILLER DE LOTE  ====================  //
    // =====================================================================  //
    private function leRegistroTraillerLote($registro){
        // CAMPO: Ocorrências
        // POSIÇÃO: 231 a 240
        // PICTURE: X(10)
        $ocorrTraillerLote = trim(substr($registro, 230, 10));

        $this->retorno->setIdsitucnabp(self::situacaoCnab($this->retorno->getOcorrencia().$ocorrTraillerLote.$this->ocorrHeader));
    }

    /**
     * @param $cdocorrencia
     * @return string
     */
    private function situacaoCnab($cdocorrencia){
        
        if($cdocorrencia){
            if(strpos($cdocorrencia, 'BD') !== false || strpos($cdocorrencia, 'BE') !== false)
                return '20'; //Retorno OK
            elseif(strpos($cdocorrencia, 'CE') !== false || strpos($cdocorrencia, 'NE') !== false)
                return '00'; //Não Agendado
            elseif(strpos($cdocorrencia, '00') !== false)
                return '90'; //Baixado
            elseif(strpos($cdocorrencia, '00') === false && strpos($cdocorrencia, 'AE') === false && strpos($cdocorrencia, 'BD') === false
                && strpos($cdocorrencia, 'BE') === false && strpos($cdocorrencia, 'CE') === false && strpos($cdocorrencia, 'IR') === false
                && strpos($cdocorrencia, 'NE') === false)
                return '01'; //Retorno com Erro
        }
        return '10'; //Sem retorno
    }
}